# Custom Entity Field Updates 

Custom Entity Field Updates is a service that allows you to implement the modification of custom entity fields in hook_update_N.
The latest version is 1.0.x

## Requirements

* PHP 8.1 or newer.
* Drupal >= 10.

For a full list of dependencies, please check the [composer.json](composer.json) file.


## HOW TO USE

### This module is intended for custom entities only.

First of all you need to go to the file under web/modules/custom/metric_group/src/Entity/<entity_class>.php and make the necessary changes to add, remove or modify fields for a specific custom entity.

Then create or modify the <custom_entity_module>.install file under web/modules/custom/<custom_entity_module>.

**web/modules/custom/<custom_entity_module>/<custom_entity_module>.install**:
```php
<?php

use Drupal\Core\Utility\UpdateException;

function <custom_entity_module>_update_100101() {
  // Inspired by https://www.drupal.org/node/2554097

  // Declare a structured array referencing fields that need to be changed. This array is
  // structured as $changesList["<entity type identifier>"]["<field identifier>"], where an element
  // at the last level (<identifier of the field>) can be just a label, or its value can be null,
  // an empty array, or an array containing the boolean element "keepData" (which defaults to true,
  // when not specified) to tell the process if it should preserve eventual existing data for that
  // field (this only works with update operations).
  $changesList = [
    "<entity_type>" => [ // Identifier for entity type "<entity_type>".
      "new_field", // Identifier for field "new_field".
      "change_size_field" => [ // Identifier for field "change_size_field".
        "length" => 512, // Change the property to a this value.
      ],
      "update_field_keep_data" => [ // Identifier for field "update_field_keep_data".
        "keepData" => false, // Do discard all existing data for this field.
      ],
      "remove_field", // Identifier for field "remove_field".
    ],
  ];

  if (\Drupal::moduleHandler()->moduleExists('custom_entity_field_updates')) {
    $message =\Drupal::service("custom_entity_field_updates.functions")->changeEntities($changesList);
  }
  else {
    $message = "Module custom_entity_field_updates not installed/enabled. \n 
    Please enable the module before installing this update";

    throw new UpdateException($message);
  }
  
  return $message;
}

```
